import asyncio
import datetime
from logging import LogRecord
import os
import socket
from typing import Any, Dict, Optional
import aiohttp
import logging_json
import logging.config

# Note: default port is 9200, not 80!
destHost = os.environ.get('VICTORIALOG_HTTP', 'http://victorialogs:80/logs')


class Formatter(logging_json.JSONFormatter):

    def __init__(self,
                 *args,
                 fields: Dict[str, Any] = {},
                 streamFields: Dict[str, str] = {},
                 message_field_name: str = "message",
                 exception_field_name: str | None = "exception",
                 **kwargs):
        super().__init__(*args,
                         fields=fields,
                         message_field_name=message_field_name,
                         exception_field_name=exception_field_name,
                         **kwargs)
        self.streamFields = streamFields

    def format(self, record: logging.LogRecord):
        record.line = f'{record.filename}:{record.lineno}'
        record._stream = self.streamFields
        record._time = datetime.datetime.now(datetime.timezone.utc).isoformat()
        return super().format(record)


class Handler(logging.Handler):

    def __init__(self, rootUrl: str, timezone: str):
        super(Handler, self).__init__()
        self.rootUrl = rootUrl
        self.q = asyncio.Queue()
        self.senderTask = None

    def emit(self, record: LogRecord):
        if self.senderTask is None:
            try:
                asyncio.get_running_loop()
            except Exception:
                return
            self.senderTask = asyncio.create_task(self.sender())
        self.q.put_nowait(record)

    async def sender(self):
        while True:
            toSend = [await self.q.get()]
            while not self.q.empty():
                toSend.append(await self.q.get())

            formatted = [self.format(r) for r in toSend]
            jsonLines = ''.join(line + '\n' for line in formatted)
            print(repr(jsonLines))
            qs = '_msg_field=message'

            async with aiohttp.ClientSession() as session:
                async with session.post(
                        self.rootUrl + 'insert/jsonline?' + qs,
                        headers={'content-type': "application/stream+json"},
                        data=jsonLines.encode('utf8')) as resp:
                    print('posted', resp.status)


def initLog(level=logging.DEBUG, stderrToo=False) -> logging.Logger:
    log = logging.getLogger()
    log.setLevel(level)

    streamFields = {'host': socket.gethostname()}

    jsonFormatter = Formatter(
        style='{',
        fields={
            "level_name": "levelname",
            "thread_name": "threadName",
            "process_name": "processName",
            "line": "line",
        },
        streamFields=streamFields,
    )

    if stderrToo:
        stderr = logging.StreamHandler()
        stderr.setFormatter(jsonFormatter)
        log.addHandler(stderr)

    victoriaHandler = Handler(rootUrl=destHost.rstrip('/') + '/',
                              timezone='America/Los_Angeles')
    victoriaHandler.setFormatter(jsonFormatter)
    log.addHandler(victoriaHandler)
    return log
